/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

/**
 * 
 * <#if locale="en">
 * <p>
 * Lifecycle of {@link Lucy} itself and components.
 * </p>
 * <#else>
 * <p>
 * {@link Lucy}とコンポーネントのライフサイクルを規定するenumです.
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public enum Lifecycle {

	/**
	 * <#if locale="en">
	 * <p>
	 * Container itself is now created.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	CONTAINER_CREATED,

	/**
	 * <#if locale="en">
	 * <p>
	 * Container configuration phase.
	 * </p>
	 * <#else>
	 * <p>
	 * {@link Lucy}自身の設定フェーズです.
	 * </p>
	 * </#if>
	 */
	CONTAINER_CONFIGURED,

	/**
	 * <#if locale="en">
	 * <p>
	 * Container preparation for components like eager singletons.
	 * </p>
	 * <#else>
	 * <p>
	 * 事前に準備が必要なeager-singletonなどのコンポーネントの準備フェーズです.
	 * </p>
	 * </#if>
	 */
	CONTAINER_PREPARED,

	/**
	 * <#if locale="en">
	 * <p>
	 * Class loading phase.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	CLASS_LOADING,

	/**
	 * <#if locale="en">
	 * <p>
	 * Component creating phase.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	COMPONENT_CREATING,

	/**
	 * <#if locale="en">
	 * <p>
	 * The phase just after components are created.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	COMPONENT_CREATED,

	/**
	 * <#if locale="en">
	 * <p>
	 * Component initializing phase.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	COMPONENT_INITIALIZE,

	/**
	 * <#if locale="en">
	 * <p>
	 * Component destroy phase.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	COMPONENT_DESTROY,

	/**
	 * <#if locale="en">
	 * <p>
	 * Container shutdown.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	CONTAINER_DESTROY;

	/**
	 * <#if locale="en">
	 * <p>
	 * Get component creation lifecycle.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	public static Lifecycle[] getComponentLifecycle() {
		return new Lifecycle[] { CLASS_LOADING, COMPONENT_CREATING,
				COMPONENT_CREATED, COMPONENT_INITIALIZE, COMPONENT_DESTROY };
	}

}
