/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import java.util.List;
import java.util.Map;

import org.t2framework.commons.meta.BeanDesc;

/**
 * 
 * <#if locale="en">
 * <p>
 * {@link LucyContext} is an context object that keeps components meta
 * information, such as binding information to some
 * {@link org.t2framework.lucy.spi.Behavior}.This enables {@link Lucy} to
 * put/get/remove operations for these information.
 * 
 * This instance is not expected to create outer of {@link Lucy}.
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * @see org.t2framework.commons.meta.BeanDesc
 * @see org.t2framework.lucy.BeanDescBag
 * @sing 0.5.2
 */
public interface LucyContext extends Iterable<ConfigBindResult> {

	<T> BeanDescBag<T> getBeanDescBag(Class<? extends T> key);

	<T> void putBeanDesc(Class<?> clazz, BeanDesc<? extends T> beanDesc);

	Map<Class<?>, BeanDescBag<?>> getAllBeanDescBags();

	<T> List<BeanDesc<T>> getBeanDescList(Class<? super T> original);

	<T> BeanDesc<T> getBeanDescByIdentifier(Object identifier);

	<T> void addNamedBeanDesc(BeanDesc<T> beanDesc);

	List<BeanDesc<?>> getNamedBeanDescList();

	<T> ConfigBindResult getConfigBindResult(BeanDesc<? extends T> beanDesc);

	<T> boolean hasConfigBindResult(BeanDesc<? extends T> beanDesc);

	<T> void putConfigBindResult(BeanDesc<? extends T> beanDesc,
			ConfigBindResult bindResult);

	<T> ConfigBindResult removeConfigBindResult(BeanDesc<? extends T> beanDesc);

	<T> void removeBeanDesc(BeanDesc<? extends T> beanDesc);

	void destroy();

}
