/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.config.stax;

import org.t2framework.commons.aop.spi.Pointcut;
import org.t2framework.commons.el.Expression;
import org.t2framework.commons.meta.ConfigContainer;
import org.t2framework.commons.util.StringUtil;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.aop.config.AspectConfig;
import org.t2framework.lucy.aop.config.impl.AspectConfigImpl;
import org.t2framework.lucy.aop.impl.PointcutImpl;
import org.t2framework.lucy.config.stax.AbstractExpressionHandler;
import org.t2framework.lucy.config.stax.Attributes;
import org.t2framework.lucy.config.stax.XmlEventContext;
import org.t2framework.lucy.config.stax.XmlEventHandler;

/**
 * <#if locale="en">
 * <p>
 * Tag handler for <aspect> tag.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 */
public class AspectTagHandler extends AbstractExpressionHandler implements
		XmlEventHandler {

	/**
	 * <#if locale="en">
	 * <p>
	 * Start tag.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	@Override
	public void start(XmlEventContext context, Attributes attributes) {
		final String pattern = attributes.getValue("pointcut");
		final Pointcut pointcut = createPointcut(pattern);
		AspectConfig aspectConfig = createAspectConfig(context.getLucy(),
				pointcut);
		context.push(aspectConfig);
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * End tag.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	@Override
	public void end(XmlEventContext context, String body) {
		AspectConfig acd = (AspectConfig) context.pop();
		String s = body.trim();
		Expression<Lucy> e = getExpression(s);
		if (e != null) {
			acd.add(e);
		}
		ConfigContainer container = (ConfigContainer) context.peek();
		container.addConfig(acd);
	}

	protected AspectConfig createAspectConfig(Lucy lucy, Pointcut pointcut) {
		return new AspectConfigImpl(lucy, pointcut);
	}

	protected Pointcut createPointcut(String pattern) {
		if (StringUtil.isEmpty(pattern) == false) {
			return new PointcutImpl(pattern);
		} else {
			return new PointcutImpl();
		}
	}
}
