/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.impl;

import java.lang.reflect.AccessibleObject;
import java.util.List;
import java.util.Map;

import javassist.Modifier;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.InterceptorDescStrategy;

/**
 * <#if locale="en">
 * <p>
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 */
public class InterceptorDescStrategyImpl implements InterceptorDescStrategy {

	protected static final Logger LOG = Logger
			.getLogger(InterceptorDescStrategy.class);

	@Override
	public <T> Map<AccessibleObject, InterceptorDesc> assemble(
			final BeanDesc<T> beanDesc, final AspectDesc aspectDesc) {
		Assertion.notNulls(beanDesc, aspectDesc);
		Map<AccessibleObject, InterceptorDesc> map = CollectionsUtil
				.newHashMap();
		assemleByPropertyDesc(map, beanDesc, aspectDesc);
		assemleByMethodDesc(map, beanDesc, aspectDesc);
		return map;
	}

	protected <T> void assemleByPropertyDesc(
			final Map<AccessibleObject, InterceptorDesc> map,
			final BeanDesc<T> beanDesc, final AspectDesc aspectDesc) {
		for (PropertyDesc<T> pd : beanDesc.getAllPropertyDesc()) {
			if (pd.isReadable()) {
				assemble0(map, pd.getReadMethodDesc(), aspectDesc);
			}
			if (pd.isWritable()) {
				assemble0(map, pd.getWriteMethodDesc(), aspectDesc);
			}
		}
	}

	protected <T> void assemleByMethodDesc(
			final Map<AccessibleObject, InterceptorDesc> map,
			final BeanDesc<T> beanDesc, final AspectDesc aspectDesc) {
		for (MethodDesc md : beanDesc.getAllMethodDesc()) {
			assemble0(map, md, aspectDesc);
		}
	}

	protected <T> void assemble0(Map<AccessibleObject, InterceptorDesc> map,
			MethodDesc methodDesc, AspectDesc aspectDesc) {
		final List<Interceptor> list = CollectionsUtil.newArrayList();
		if (methodDesc == null) {
			return;
		}
		for (Aspect a : aspectDesc.getAspects()) {
			LOG.log("DLucyAop0005", new Object[] {
					methodDesc.getDeclaringClass().getName(),
					methodDesc.getMethodName(),
					a.getPointcut().getPatternString() });
			if (a.getPointcut().isApplied(methodDesc.getMethod())) {
				list.addAll(a.getInterceptors());
				LOG.log("DLucyAop0006", new Object[] {
						methodDesc.getDeclaringClass().getName(),
						methodDesc.getMethodName() });
			} else {
				LOG.log("DLucyAop0007", new Object[] {
						methodDesc.getDeclaringClass().getName(),
						methodDesc.getMethodName() });
			}
		}
		final int mod = methodDesc.getModifiers();
		if (list.size() == 0 || Modifier.isFinal(mod)) {
			return;
		}
		map
				.put(methodDesc.getMethod(), createInterceptorDesc(methodDesc,
						list));
	}

	protected InterceptorDesc createInterceptorDesc(MethodDesc methodDesc,
			List<Interceptor> list) {
		return new InterceptorDescImpl(methodDesc, list);
	}

}
