/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.impl;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.regex.Pattern;

import org.t2framework.commons.aop.spi.Pointcut;
import org.t2framework.commons.util.Assertion;

public class PointcutImpl implements Pointcut, Serializable {

	static final long serialVersionUID = 0L;

	protected String regex;

	protected Pattern pattern;

	public PointcutImpl() {
		this(DEFAULT_REGEX);
	}

	public PointcutImpl(String regexes) {
		this.regex = Assertion.notNull(regexes);
		this.pattern = Pattern.compile(regexes);
	}

	@Override
	public Pattern getPattern() {
		return this.pattern;
	}

	protected boolean match(final String methodName) {
		if (this.pattern.matcher(methodName).matches()) {
			return true;
		}
		return false;
	}

	@Override
	public String getPatternString() {
		return regex;
	}

	@Override
	public boolean isApplied(Method method) {
		final String methodName = Assertion.notNull(method).getName();
		return match(methodName);
	}

}
