/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.javassist.behavior;

import java.util.List;

import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.aop.AspectAssembler;
import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.InstanceFactory;
import org.t2framework.commons.meta.impl.InstanceFactoryImpl;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.aop.behavior.AbstractAspectBehavior;
import org.t2framework.lucy.aop.config.AspectConfig;
import org.t2framework.lucy.aop.impl.AspectAssemblerImpl;

/**
 * <#if locale="en">
 * <p>
 * JavassistAspectBehavior is an implementation for Javassist base AOP.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author c9katayama
 * 
 */
@ConfigurationTarget( { org.t2framework.commons.annotation.core.Aspect.class,
		AspectConfig.class })
public class JavassistAspectBehavior extends AbstractAspectBehavior {

	protected AspectAssembler<Lucy> aspectAssembler;

	protected InstanceFactory<Object> instanceFactory;

	@Override
	public void init() {
		super.init();
		aspectAssembler = new AspectAssemblerImpl<Lucy>();
		instanceFactory = new InstanceFactoryImpl<Object>();
	}

	@SuppressWarnings("unchecked")
	public JavassistAspectBehavior(BeanDesc beanDesc, ClassDesc classDesc,
			Config cd) {
		super(beanDesc, classDesc, cd);
	}

	@Override
	protected AspectAssembler<Lucy> createAspectAssembler() {
		return aspectAssembler;
	}

	@Override
	protected InstanceFactory<Object> createInstanceFactory(List<Aspect> aspects) {
		return instanceFactory;
	}

	@Override
	public <T> void destroy() {
		super.destroy();
		aspectAssembler = null;
		instanceFactory = null;
	}
}
