/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.proxy.behavior;

import java.lang.reflect.AccessibleObject;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.aop.AspectAssembler;
import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.InstanceFactory;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.InterceptorDescStrategy;
import org.t2framework.lucy.aop.behavior.AbstractAspectBehavior;
import org.t2framework.lucy.aop.config.AspectConfig;
import org.t2framework.lucy.aop.impl.AspectAssemblerImpl;
import org.t2framework.lucy.aop.impl.AspectDescImpl;
import org.t2framework.lucy.aop.proxy.ProxyInterceptorDescStrategyImpl;
import org.t2framework.lucy.aop.proxy.meta.impl.ProxyInstanceFactoryImpl;

/**
 * <#if locale="en">
 * <p>
 * ProxyAspectBehavior is an implementation for proxy based AOP.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author c9katayama
 */
@ConfigurationTarget( { org.t2framework.commons.annotation.core.Aspect.class,
		AspectConfig.class })
public class ProxyAspectBehavior extends AbstractAspectBehavior {

	protected AspectAssembler<Lucy> aspectAssembler;

	@SuppressWarnings("unchecked")
	public ProxyAspectBehavior(BeanDesc beanDesc, ClassDesc classDesc, Config cd) {
		super(beanDesc, classDesc, cd);
	}

	@Override
	public void init() {
		super.init();
		aspectAssembler = new AspectAssemblerImpl<Lucy>();
	}

	@Override
	protected AspectAssembler<Lucy> createAspectAssembler() {
		return aspectAssembler;
	}

	@Override
	protected InstanceFactory<Object> createInstanceFactory(List<Aspect> aspects) {
		BeanDesc<Object> beanDesc = getBeanDesc();
		AspectDesc aspectDesc = new AspectDescImpl(
				beanDesc.getComponentClass(), aspects);
		aspectDesc.setEnhancedClass(beanDesc.getEnhancedComponentClass());
		InterceptorDescStrategy strategy = createInterceptorDescStrategy();
		Map<AccessibleObject, InterceptorDesc> interceptorMap = strategy
				.assemble(beanDesc, aspectDesc);
		return new ProxyInstanceFactoryImpl<Object>(interceptorMap);
	}

	protected InterceptorDescStrategy createInterceptorDescStrategy() {
		return new ProxyInterceptorDescStrategyImpl();
	}

	@Override
	public void destroy() {
		super.destroy();
		aspectAssembler = null;
	}
}
