/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.util;

import static org.t2framework.lucy.aop.AopConstants.*;

import java.util.Set;

import org.t2framework.commons.util.CollectionsUtil;

/**
 * <#if locale="en">
 * <p>
 * Aop utility class.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author c9katayama
 * @author shot
 */
public class AopUtil {

	protected static final Set<String> enhancedClassNames = CollectionsUtil
			.newConcurrentHashSet();

	public static boolean javassistDisabled = false;

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * Create enhanced class name.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param baseClass
	 * @return
	 */
	public static String createEnhancedClassName(Class<?> baseClass) {
		final StringBuffer buf = new StringBuffer(200);
		final String targetClassName = baseClass.getName();
		final Package pkg = baseClass.getPackage();
		if (targetClassName.startsWith("java.")
				|| (pkg != null && pkg.isSealed())) {
			buf.append(PREFIX_ENHANCED_CLASS);
		}
		buf.append(targetClassName).append(SUFFIX_ENHANCED_CLASS).append(
				Integer.toHexString(new Object().hashCode()));
		final int length = buf.length();
		for (int i = 0; enhancedClassNames.contains(new String(buf)); ++i) {
			buf.setLength(length);
			buf.append("_").append(i);
		}
		String enhancedClassName = new String(buf);
		enhancedClassNames.add(enhancedClassName);
		return enhancedClassName;
	}

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * True if javassist is used for weaving.
	 * </p>
	 * <#else>
	 * <p>
	 * Javassistが有効かどうかを判定します.
	 * </p>
	 * </#if>
	 * 
	 * @return boolean
	 */
	public static boolean isJavassistEnabled() {
		if (javassistDisabled) {
			return false;
		}
		try {
			Class.forName("javassist.ClassPool");
			return true;
		} catch (ClassNotFoundException ignore) {
			return false;
		}
	}

}
