/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.behavior;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.inject.ConstructorInjector;
import org.t2framework.lucy.inject.Injector;
import org.t2framework.lucy.inject.SimpleConstructorInjector;
import org.t2framework.lucy.spi.AbstractBehavior;
import org.t2framework.lucy.spi.Behavior;

/**
 * 
 * <#if locale="en">
 * <p>
 * {@link AbstractComponentCreationBehavior} is a behavior for creating
 * component.
 * </p>
 * <#else>
 * <p>
 * コンポーネントの作成を実施するタイプの{@link Behavior}の共通抽象クラスです．
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public abstract class AbstractComponentCreationBehavior extends
		AbstractBehavior {

	protected Lock lock = new ReentrantLock();

	protected final Injector injector;

	public AbstractComponentCreationBehavior(BeanDesc<?> beanDesc,
			ClassDesc<?> classDesc, Config cd) {
		super(beanDesc, classDesc, cd);
		if (beanDesc.getConstructorDesc().hasOnlyDefaultConstructor()) {
			injector = new SimpleConstructorInjector(beanDesc);
		} else {
			injector = new ConstructorInjector(beanDesc);
		}
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * Execute component creation.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	@Override
	public <T> T execute(T t, Lucy lucy) {
		lock.lock();
		try {
			t = handleComponentCreation(t, lucy, getBeanDesc());
		} finally {
			lock.unlock();
		}
		return t;
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * Create and inject component.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param <T>
	 * @param t
	 * @param lucy
	 * @param beanDesc
	 * @return T
	 */
	protected abstract <T> T handleComponentCreation(T t, Lucy lucy,
			BeanDesc<Object> beanDesc);

	@Override
	public final Lifecycle getLifecycle() {
		return Lifecycle.COMPONENT_CREATING;
	}

	protected Injector getInjector() {
		return injector;
	}
}
