/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.meta;

import java.util.LinkedList;
import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.ConstructorDesc;
import org.t2framework.commons.meta.ExpressionAware;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.meta.impl.BeanDescImpl;
import org.t2framework.commons.util.Assertion;

/**
 * 
 * <#if locale="en">
 * <p>
 * {@link ConfigurableBeanDesc} is the specialized {@link BeanDesc} for external
 * xml configuration.
 * </p>
 * <#else>
 * <p>
 * 設定ファイル経由で生成される特殊な{@link BeanDesc}です．{@link BeanDesc}生成後に {@link MethodDesc}や
 * {@link PropertyDesc}の置き換えなどを行います．
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 * @param <T>
 */
public class ConfigurableBeanDesc<T> extends BeanDescImpl<T> {

	protected LinkedList<ExpressionAware> latests;

	protected String autoinject;

	protected String path;

	protected int lineNumber;

	protected int columnNumber;

	public ConfigurableBeanDesc(Class<? extends T> componentClass) {
		super(componentClass);
	}

	public ConfigurableBeanDesc(ClassDesc<T> classDesc) {
		super(classDesc);
	}

	@Override
	protected void init0() {
		super.init0();
		this.latests = new LinkedList<ExpressionAware>();
	}

	@SuppressWarnings("unchecked")
	public <D extends ExpressionAware> D getLatest() {
		return (D) latests.pollFirst();
	}

	@Override
	public void addMethodDesc(MethodDesc md) {
		final String methodName = md.getMethodName();
		List<MethodDesc> list = methodDescMap.get(methodName);
		if (list == null) {
			super.addMethodDesc(md);
			return;
		}
		if (replaceMethodDesc(list, md) == false) {
			super.addMethodDesc(md);
		}
		replaceMethodDesc(getAllMethodDesc(), md);
		latests.offerFirst(md);
	}

	protected boolean replaceMethodDesc(List<MethodDesc> list, MethodDesc desc) {
		for (int i = 0; i < list.size(); ++i) {
			MethodDesc md = list.get(i);
			if (md.equals(desc)) {
				list.remove(i);
				list.add(i, desc);
				return true;
			}
		}
		return false;
	}

	@Override
	public void addPropertyDesc(PropertyDesc<T> pd) {
		Assertion.notNull(pd);
		final String propertyName = pd.getPropertyName();
		PropertyDesc<T> propertyDesc = propertyDescMap.get(propertyName);
		if (propertyDesc == null) {
			super.addPropertyDesc(pd);
			return;
		}
		propertyDescMap.put(propertyName, pd);
		replacePropertyDesc(propertyDescList, pd);
		latests.offerFirst(pd);
	}

	protected boolean replacePropertyDesc(List<PropertyDesc<T>> list,
			PropertyDesc<T> desc) {
		for (int i = 0; i < list.size(); ++i) {
			PropertyDesc<T> pd = list.get(i);
			if (pd.getPropertyName().equals(desc.getPropertyName())) {
				list.remove(i);
				list.add(i, desc);
				return true;
			}
		}
		return false;
	}

	@Override
	public void setConstructorDesc(ConstructorDesc<T> constructorDesc) {
		super.setConstructorDesc(constructorDesc);
		latests.offerFirst(constructorDesc);
	}

	public String getAutoinject() {
		return autoinject;
	}

	public void setAutoinject(String autoinject) {
		this.autoinject = autoinject;
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public int getLineNumber() {
		return lineNumber;
	}

	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}

	public int getColumnNumber() {
		return columnNumber;
	}

	public void setColumnNumber(int columnNumber) {
		this.columnNumber = columnNumber;
	}

}
