/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.meta;

import java.lang.annotation.Annotation;
import java.util.Map;

import org.t2framework.commons.annotation.composite.PrototypeScope;
import org.t2framework.commons.annotation.composite.SingletonScope;
import org.t2framework.commons.exception.NoSuchInstaceConfigRuntimeException;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.config.meta.impl.LazySingletonInstanceConfig;
import org.t2framework.lucy.config.meta.impl.PrototypeInstanceConfig;
import org.t2framework.lucy.config.meta.impl.SingletonInstanceConfig;

/**
 * 
 * <#if locale="en">
 * <p>
 * Instance creating factory.
 * </p>
 * <#else>
 * <p>
 * インスタンス生成するための{@link InstanceConfig}を取得するためのファクトリです．
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class InstanceConfigFactory {

	public static final SingletonInstanceConfig SINGLETON_CONFIG = new SingletonInstanceConfig();

	public static final PrototypeInstanceConfig PROTOTYPE_CONFIG = new PrototypeInstanceConfig();

	public static final LazySingletonInstanceConfig LAZY_SINGLETON_CONFIG = new LazySingletonInstanceConfig();

	public static final String DEFAULT = SINGLETON_CONFIG.getName();

	protected static final Map<String, InstanceConfig> scopeStringMap = CollectionsUtil
			.newHashMap();

	protected static final Map<Class<? extends Annotation>, InstanceConfig> scopeAnnotationMap = CollectionsUtil
			.newHashMap();

	static {
		scopeStringMap.put(SINGLETON_CONFIG.getName(), SINGLETON_CONFIG);
		scopeStringMap.put(PROTOTYPE_CONFIG.getName(), PROTOTYPE_CONFIG);
		scopeStringMap.put(LAZY_SINGLETON_CONFIG.getName(),
				LAZY_SINGLETON_CONFIG);

		scopeAnnotationMap.put(SingletonScope.class, SINGLETON_CONFIG);
		scopeAnnotationMap.put(PrototypeScope.class, PROTOTYPE_CONFIG);
	}

	public static InstanceConfig getInstanceConfig(String type) {
		Assertion.notNull(type);
		InstanceConfig instanceConfig = scopeStringMap.get(type);
		if (instanceConfig == null) {
			throw new NoSuchInstaceConfigRuntimeException(type);
		}
		return instanceConfig;
	}

	public static InstanceConfig getInstanceConfig(
			Class<? extends Annotation> annotationClass) {
		Assertion.notNull(annotationClass);
		InstanceConfig instanceConfig = scopeAnnotationMap.get(annotationClass);
		if (instanceConfig == null) {
			throw new NoSuchInstaceConfigRuntimeException(annotationClass
					.getName());
		}
		return instanceConfig;
	}

	public static void addInstanceConfig(
			Class<? extends Annotation> annotationClass,
			InstanceConfig instanceConfig) {
		Assertion.notNull(annotationClass);
		Assertion.notNull(instanceConfig);
		scopeAnnotationMap.put(annotationClass, instanceConfig);
	}

	public static void addInstanceConfig(InstanceConfig instanceConfig) {
		Assertion.notNull(instanceConfig);
		scopeStringMap.put(instanceConfig.getName(), instanceConfig);
	}

}
