/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.config.meta.ConfigurableBeanDesc;

/**
 * 
 * <#if locale="en">
 * <p>
 * Root tag handler(<lucy-config>) for lucy configuration.
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class LucyConfigTagHandler implements XmlEventHandler {

	protected static Logger logger = Logger
			.getLogger(LucyConfigTagHandler.class);

	@Override
	public void start(XmlEventContext context, Attributes attributes) {
		context.clear();
		LucyConfig config = new LucyConfig();
		context.push(config);
	}

	@Override
	public void end(final XmlEventContext context, final String body) {
		final LucyConfig config = (LucyConfig) context.pop();
		final Lucy lucy = context.getLucy();
		ConfigurableBeanDesc<?>[] beanDescs = config.getBeanDescs();
		for (ConfigurableBeanDesc<?> bd : beanDescs) {
			logRegistration(context, bd);
			setSource(bd);
			lucy.register(bd);
		}
	}

	protected void logRegistration(final XmlEventContext context,
			final ConfigurableBeanDesc<?> bd) {
		if (bd != null) {
			final Class<?> componentClass = bd.getComponentClass();
			final String className = componentClass.getName();
			final String path = bd.getPath();
			final int lineNumber = bd.getLineNumber();
			final int columnNumber = bd.getColumnNumber();
			if (bd.hasName()) {
				logger
						.log("DLucyCore0004", new Object[] { className,
								bd.getComponentName(), path, lineNumber,
								columnNumber });
			} else {
				logger.log("DLucyCore0005", new Object[] { className, path,
						lineNumber, columnNumber });
			}
		}
	}

	protected void setSource(ConfigurableBeanDesc<?> bd) {
		Map<String, String> source = new HashMap<String, String>();
		source.put("path", bd.getPath());
		source.put("lineNumber", String.valueOf(bd.getLineNumber()));
		source.put("columnNumber", String.valueOf(bd.getColumnNumber()));
		bd.setSource(source);
	}

	public static class LucyConfig {

		protected List<ConfigurableBeanDesc<?>> beanDescList = CollectionsUtil
				.newArrayList();

		public void addBeanDesc(ConfigurableBeanDesc<?> beanDesc) {
			beanDescList.add(Assertion.notNull(beanDesc));
		}

		public ConfigurableBeanDesc<?>[] getBeanDescs() {
			return beanDescList.toArray(new ConfigurableBeanDesc<?>[0]);
		}

	}
}
