/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.t2framework.commons.exception.IORuntimeException;
import org.t2framework.commons.util.Logger;
import org.t2framework.commons.util.MessageFormatter;
import org.t2framework.commons.util.ResourceUtil;
import org.t2framework.commons.util.StringUtil;

/**
 * <#if locale="en">
 * <p>
 * <properties> tag handler.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class PropertiesTagHandler implements XmlEventHandler {

	public static final String NAME_ATTRIBUTE = "name";

	public static final String CLASS_NAME = Properties.class.getName();

	protected static Logger logger = Logger
			.getLogger(PropertiesTagHandler.class);

	@Override
	public void start(XmlEventContext context, Attributes attributes) {
		final String name = attributes.getAndRemoveValue(NAME_ATTRIBUTE);
		if (StringUtil.isEmpty(name)) {
			final String message = MessageFormatter.getMessage("ELucyCore0014",
					new Object[] { "properties", context.getLineNumber(),
							context.getColumnNumber(), "name" });
			throw new IllegalStateException(message);
		}
		context.push(name);
	}

	@Override
	public void end(XmlEventContext context, String body) {
		String name = (String) context.pop();
		if (StringUtil.isEmpty(body)) {
			final String message = MessageFormatter.getMessage("ELucyCore0015",
					new Object[] { "properties", context.getLineNumber(),
							context.getColumnNumber() });
			throw new IllegalStateException(message);
		}
		InputStream is = ResourceUtil.getResourceAsStream(body);
		Properties prop = new Properties();
		try {
			prop.load(is);
		} catch (IOException e) {
			throw new IORuntimeException(e);
		}
		logger.log("DLucyCore0004", new Object[] { CLASS_NAME, name,
				context.getPath(), context.getLineNumber(),
				context.getColumnNumber() });
		context.getLucy().register(prop, name);
	}

}
