/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import java.util.Stack;

import javax.xml.stream.Location;
import javax.xml.stream.events.XMLEvent;

import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.StringUtil;
import org.t2framework.lucy.Lucy;

/**
 * <#if locale="en">
 * <p>
 * A context object for parsing lucy configuration xml file.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class XmlEventContext {

	protected Stack<String> bodyStack = CollectionsUtil.newStack();

	protected Stack<Object> stack = CollectionsUtil.newStack();

	protected XMLEvent currentEvent;

	protected final Lucy lucy;

	protected String path;

	public XmlEventContext(Lucy lucy) {
		this.lucy = Assertion.notNull(lucy);
	}

	public String peekBody() {
		return bodyStack.peek();
	}

	public String popBody() {
		return bodyStack.pop();
	}

	public void pushBody(String body) {
		if (!StringUtil.isEmpty(body)) {
			bodyStack.push(body);
		}
	}

	public void setCurrentEvent(XMLEvent event) {
		this.currentEvent = event;
	}

	public void clear() {
		bodyStack.clear();
		stack.clear();
		currentEvent = null;
	}

	public void push(Object object) {
		stack.push(object);
	}

	public Object pop() {
		return stack.pop();
	}

	public Object peek() {
		return stack.peek();
	}

	public Lucy getLucy() {
		return lucy;
	}

	public int getLineNumber() {
		return getLocation().getLineNumber();
	}

	public int getColumnNumber() {
		return getLocation().getColumnNumber();
	}

	protected Location getLocation() {
		return currentEvent.getLocation();
	}

	public void startHandle(String path) {
		this.path = path;
	}

	public String getPath() {
		return path;
	}

	public void endHandle() {
		this.lucy.prepare();
		clear();
	}
}
