/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.impl;

import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.exception.TooManyRegistrationException;

/**
 * <#if locale="en">
 * <p>
 * DefaultBeanDescStrategyImpl is a strategy class of BeanDescFactory.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 */
public class DefaultBeanDescStrategyImpl implements BeanDescStrategy {

	protected final Lucy lucy;

	public DefaultBeanDescStrategyImpl(Lucy lucy) {
		this.lucy = lucy;
	}

	@Override
	public <T> BeanDesc<T> getBeanDesc(Class<T> beanClass,
			List<BeanDesc<T>> list) {
		Assertion.notNull(beanClass);
		Assertion.notNull(list);
		if (list.size() == 1) {
			return list.get(0);
		} else {
			throw new TooManyRegistrationException(beanClass);
		}
	}

	@Override
	public <T> T handleNoBeanDesc(Object key) {
		return null;// if no beanDesc, simply return null.
	}

}
