/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.impl;

import org.t2framework.commons.bootstrap.CommonsInitializer;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.module.Module;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.ComponentBuilderFactory;
import org.t2framework.lucy.ConfigBinder;
import org.t2framework.lucy.ConfigBinderFactory;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyBootstrap;
import org.t2framework.lucy.LucyComponentRegistrar;
import org.t2framework.lucy.module.LucyAopModule;
import org.t2framework.lucy.module.LucyCoreModule;
import org.t2framework.lucy.module.impl.LucyCoreModuleImpl;
import org.t2framework.lucy.spi.ComponentBuilder;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * 
 * <#if locale="en">
 * <p>
 * Base Lucy configuration.
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class DefaultLucyConfiguration implements LucyConfiguration {

	protected final ConfigBinder rootBinder;

	protected LucyComponentRegistrar<?> componentRegistrar;

	protected LucyCoreModule coreModule = new LucyCoreModuleImpl();

	public DefaultLucyConfiguration() {
		this(createDefaultConfigBinder());
	}

	public DefaultLucyConfiguration(ConfigBinder binder) {
		this(Assertion.notNull(binder), (Module[]) null);
	}

	public DefaultLucyConfiguration(Module... modules) {
		this(createDefaultConfigBinder(), modules);
	}

	public DefaultLucyConfiguration(ConfigBinder binder, Module... modules) {
		if (modules != null && modules.length > 0) {
			LucyBootstrap.addModule(modules);
		}
		LucyAopModule aopModule = CommonsInitializer.get(LucyAopModule.class);
		if (aopModule != null) {
			LucyBootstrap.addModule(aopModule);
		}
		LucyBootstrap.addModule(coreModule);
		ConfigBinderFactory.setConfigBinder(binder);
		rootBinder = ConfigBinderFactory.getConfigBinder(this);
		componentRegistrar = new XMLLucyComponentRegistrar();
	}

	@Override
	public ConfigBinder loadConfigBinder() {
		return rootBinder;
	}

	@Override
	public ComponentBuilder createBuilder(BeanDesc<?> beanDesc, Lucy lucy) {
		return ComponentBuilderFactory.createBuilder(beanDesc, lucy, this);
	}

	@Override
	public BeanDescStrategy getBeanDescStrategy(Lucy lucy) {
		return new DefaultBeanDescStrategyImpl(lucy);
	}

	@Override
	@SuppressWarnings("unchecked")
	public <R> LucyComponentRegistrar<R> getAcceptableComponentRegistrar(
			R resource) {
		if (componentRegistrar.accepts(resource)) {
			return (LucyComponentRegistrar<R>) componentRegistrar;
		} else {
			return null;
		}
	}

	protected static ConfigBinder createDefaultConfigBinder() {
		return new ConfigBinderFactory.DefaultConfigBinderImpl();
	}
}
