/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.inject;

import java.lang.annotation.Annotation;

import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.ConfigContainer;
import org.t2framework.lucy.annotation.core.Inject;
import org.t2framework.lucy.config.meta.InjectConfig;
import org.t2framework.lucy.inject.AbstractInjector.InjectInfo;

/**
 * 
 * <#if locale="en">
 * <p>
 * {@link InjectorUtil} is an utility class for injection.
 * </p>
 * <#else>
 * <p>
 * {@link InjectorUtil}はインジェクションするときのユーティリティクラスです.
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class InjectorUtil {

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * &#064;Injectが{@code ConfigContainer}内に存在するかどうかチェックします.
	 * </p>
	 * </#if>
	 * 
	 * @param container
	 * @return
	 */
	public static boolean hasInjectAnnotation(ConfigContainer container) {
		if (container == null) {
			return false;
		}
		return container.hasConfig(Inject.class);
	}

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * {@code InjectConfig}が{@code ConfigContainer}内に存在するかチェックします.
	 * </p>
	 * </#if>
	 * 
	 * @param container
	 * @return
	 */
	public static boolean hasInjectConfiguration(ConfigContainer container) {
		if (container == null) {
			return false;
		}
		return container.hasConfig(InjectConfig.class);
	}

	public static boolean isInjectAnnotation(Config config) {
		if (config == null || config.hasAnnotation() == false) {
			return false;
		}
		final Class<? extends Annotation> annotationType = config
				.getAnnotation().annotationType();
		return annotationType == Inject.class;
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * Get {@link Class}[] from {@link Inject}.types() or {@link InjectConfig}
	 * .getParameterTypes().
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param container
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Class[] getTypes(ConfigContainer container) {
		if (hasInjectAnnotation(container) == false
				&& hasInjectConfiguration(container) == false) {
			return null;
		}
		Class[] types = null;
		for (Config config : container) {
			Class<?> type = config.getType();
			if (type.equals(Inject.class)) {
				types = ((Inject) config.getAnnotation()).types();
				if (types.length == 1 && types[0] == Object.class) {
					types = null;
				}
				return types;
			} else if (type.equals(InjectConfig.class)) {
				return ((InjectConfig) config).getParameterTypes();
			}
		}
		return types;
	}

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * Get inject-able component from {@code LucyHandler}.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param <T>
	 * @param type
	 * @param info
	 * @param namesIndex
	 * @param handler
	 * @return
	 */
	public static <T> T getInjectableComponent(Class<? super T> type,
			InjectInfo info, int namesIndex, InjectHandler<T> handler) {
		T value = null;
		if (isSimplyByType(info)) {
			value = handler.getFromType(type);
		} else {
			int namesLength = info.getNames().length;
			if (namesIndex < namesLength) {
				value = handler.getFromIdentifier(info.getNames()[namesIndex]);
			} else {
				value = handler.getFromType(type);
			}
		}
		return value;
	}

	private static boolean isSimplyByType(InjectInfo info) {
		return info == null || info.hasNames() == false;
	}

	/**
	 * 
	 * <#if locale="en">
	 * <p>
	 * An interface for somehow looking component from Lucy.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @author shot
	 * 
	 * @param <T>
	 */
	public static interface InjectHandler<T> {
		@SuppressWarnings("unchecked")
		T getFromType(Class type);

		T getFromIdentifier(Object identifier);
	}
}
