/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.Lifecycle;

/**
 * <#if locale="en">
 * <p>
 * Convenient abstract Behavior class.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 */
public abstract class AbstractBehavior implements Behavior {

	protected BeanDesc<Object> beanDesc;

	protected ClassDesc<Object> classDesc;

	protected MethodDesc methodDesc;

	protected PropertyDesc<Object> propertyDesc;

	protected Config config;

	/**
	 * <#if locale="en">
	 * <p>
	 * Constructors are called by reflection.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param beanDesc
	 * @param classDesc
	 * @param cd
	 */
	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, ClassDesc classDesc, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.classDesc = Assertion.notNull(classDesc);
		this.config = cd;
	}

	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, MethodDesc md, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.methodDesc = Assertion.notNull(md);
		this.config = Assertion.notNull(cd);
	}

	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, PropertyDesc pd, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.propertyDesc = Assertion.notNull(pd);
		this.config = Assertion.notNull(cd);
	}

	@Override
	public <T> void init() {
	}

	@Override
	public <T> void destroy() {
		beanDesc = null;
		classDesc = null;
		methodDesc = null;
		propertyDesc = null;
	}

	protected final BeanDesc<Object> getBeanDesc() {
		return beanDesc;
	}

	protected final ClassDesc<Object> getClassDesc() {
		return classDesc;
	}

	protected final MethodDesc getMethodDesc() {
		return methodDesc;
	}

	protected final PropertyDesc<Object> getPropertyDesc() {
		return propertyDesc;
	}

	protected final Config getConfig() {
		return config;
	}

	@Override
	public boolean isApplied(Lifecycle current) {
		return current != null && current == getLifecycle();
	}

}
