/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;

/**
 * <#if locale="en">
 * <p>
 * Behavior is base interface deciding how Lucy behaves.The behavior class
 * always go with an configuration point just like InjectBehavior is for
 * &#064;Inject or InjectConfig.
 * 
 * <pre>
 * Behavior's constructor must have at least one of these:
 *  - {@link BeanDesc}, {@link ClassDesc}, {@link Config}
 *  - {@link BeanDesc}, {@link MethodDesc}, {@link Config}
 *  - {@link BeanDesc}, {@link PropertyDesc}, {@link Config}
 * </pre>
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 */
public interface Behavior {

	/**
	 * <#if locale="en">
	 * <p>
	 * Initialize this {@link Behavior}.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param <T>
	 */
	<T> void init();

	/**
	 * <#if locale="en">
	 * <p>
	 * Execute {@link Behavior} with the instance representing T, and
	 * {@link Lucy}.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param <T>
	 * @param t
	 * @param lucy
	 * @return
	 */
	<T> T execute(T t, Lucy lucy);

	/**
	 * <#if locale="en">
	 * <p>
	 * Destroy this {@link Behavior}.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param <T>
	 */
	<T> void destroy();

	/**
	 * <#if locale="en">
	 * <p>
	 * Get {@link Lifecycle} that tells when to invoke this {@link Behavior}.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @return
	 */
	Lifecycle getLifecycle();

	/**
	 * <#if locale="en">
	 * <p>
	 * True if this {@link Behavior} should be invoked with the given
	 * {@link Lifecycle}.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @param current
	 * @return
	 */
	boolean isApplied(Lifecycle current);
}
