/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.ut;

import org.t2framework.commons.ut.DefaultUtPlugin;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyBootstrap;
import org.t2framework.lucy.impl.DefaultLucyConfiguration;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * <#if locale="en">
 * <p>
 * Test case plugin for {@link LucyTestCase}.
 * 
 * </p>
 * <#else>
 * <p>
 * 
 * </p>
 * </#if>
 * 
 * @author shot
 * 
 */
public class LucyUtPlugin extends DefaultUtPlugin {

	protected LucyConfiguration configuration;

	protected Object componentDefinition;

	public LucyUtPlugin(LucyTestCase testCase) {
		this(testCase, testCase.getLucyConfiguration());
	}

	public LucyUtPlugin(LucyTestCase testCase, LucyConfiguration configuration) {
		this(testCase, configuration, null);
	}

	public LucyUtPlugin(LucyTestCase testCase, LucyConfiguration configuration,
			Object componentDefinition) {
		super(testCase);
		this.configuration = configuration;
		this.componentDefinition = componentDefinition;
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * Begin test processing.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	@Override
	public void begin() throws Throwable {
		final LucyConfiguration configuration = getConfiguration();
		if (configuration == null) {
			throw new NullPointerException(
					"lucy configuration must not be null.");
		}
		LucyBootstrap.init(configuration, getComponentDefinition());
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * End test processing.
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 */
	@Override
	public void end() throws Throwable {
		LucyBootstrap.destroy();
	}

	/**
	 * <#if locale="en">
	 * <p>
	 * Get {@link Lucy} for unit testing.
	 * 
	 * </p>
	 * <#else>
	 * <p>
	 * 
	 * </p>
	 * </#if>
	 * 
	 * @return
	 */
	public Lucy getLucy() {
		return LucyBootstrap.getLucy();
	}

	protected LucyConfiguration getConfiguration() {
		return (configuration != null) ? configuration
				: new DefaultLucyConfiguration();
	}

	protected Object getComponentDefinition() {
		return componentDefinition;
	}
}
